#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

void numOut(uint8_t digit, uint8_t number);
void blank(void);
inline uint8_t button(void);
void init(void);
void increaseTime(uint8_t minutes);

volatile uint8_t h,m,s,state,digitstate;

#define DUTY 0.1           //LED duty cycle, max 0.25!!
#define FREQ 1000          //LED frequency
#define SET_DELAY 250      //Delay for time step increase while pressing set button
#define NEXT_SPEED_TICKS 5 //Number of time steps until next set speed


#define LED_ON_TICKS (DUTY*1000000/FREQ)/16	         //16µs/Tick
#define LED_OFF_TICKS ((1-4*DUTY)*1000000/FREQ)/16   //16µs/Tick

#define STATE_NORMAL	0
#define STATE_DEBOUNCE	1
#define STATE_SETSPEED1	2
#define STATE_SETSPEED2	3
#define STATE_SETSPEED3	4
#define STATE_INITIAL	STATE_NORMAL

#define DSTATE_1	0
#define DSTATE_2	1
#define DSTATE_3	2
#define DSTATE_4	3
#define DSTATE_OFF	4
#define DSTATE_INITIAL DSTATE_1


int main(void){
	uint8_t temp=0;	//General purpose temporary variable

	init();
	
	while(1){
		switch(state){
			case STATE_NORMAL:
				if(button())
					state=STATE_DEBOUNCE;	
				break;
				
			case STATE_DEBOUNCE:
				_delay_ms(50);
				if(button())
					state=STATE_SETSPEED1;
				else
					state=STATE_NORMAL;
				break;
				
			case STATE_SETSPEED1:
				increaseTime(1);
				_delay_ms(SET_DELAY);
				if(!button()){
					state=STATE_NORMAL;
				}else{
					if((++temp)==5){
						state=STATE_SETSPEED2;
						temp=0;
					}
				}
				break;
				
			case STATE_SETSPEED2:
				increaseTime(10);
				_delay_ms(SET_DELAY);
				if(!button()){
					state=STATE_NORMAL;
				}else{
					if((++temp)==5){
						state=STATE_SETSPEED3;
						temp=0;
					}
				}
				break;
				
			case STATE_SETSPEED3:
				increaseTime(60);
				_delay_ms(SET_DELAY);
				if(!button()){
					state=STATE_NORMAL;
				}
				break;
				
		}
		
		
	}
	return 0;
}

void init(){
	h=0;
	m=0;
	s=0;
	
	state=STATE_INITIAL;
	digitstate=DSTATE_INITIAL;
	
	//Timer 1: Clock Timer
	OCR1A=62494; //1s, adapted @Prescaler 256
	TCCR1A=0;
	TCCR1B=(1<<WGM12)|(1<<CS12); //CTC Mode, prescaler 256
	TCNT1=0;
	TIMSK=(1<<OCIE1A);
	
	//Timer 0: Digit Timer
	TCCR0A=(1<<WGM01);				//CTC Mode
	TCCR0B=(1<<CS02);				//16µs/Tick
	TCNT0=0;
	OCR0A=LED_ON_TICKS;
	TIMSK|=(1<<OCIE0A);
	
	
	DDRB=0xFF;
	blank();
	
	DDRD=(1<<PD6);					//PD6: Mittlere LEDs
	PORTD=(1<<PD6)|(1<<PD2);
	
	sei();
}


void increaseTime(uint8_t minutes){
	m+=minutes;
	h=(h+(m/60))%24;
	m%=60;
}

void numOut(uint8_t digit, uint8_t number){
	if (digit>3) {
		blank();
		return;
	}
	PORTB=number; 							//PB0-PB3 setzen -> Zahl
	PORTB= (PORTB&0x0F) | (1<<(4+digit));	//PB4-PB7 setzen -> Ziffer
}

void blank(){
	PORTB=0;
}

inline uint8_t button(){
	return (PIND & (1<<PD2))?0:1;
}

ISR(TIMER1_COMPA_vect){
	if((++s)==60){
		s=0;
		increaseTime(1);
	}
	
}

ISR(TIMER0_COMPA_vect){
	switch(digitstate){
		case DSTATE_1:
			PORTD|=(1<<PD6);
			OCR0A=LED_ON_TICKS;
			numOut(digitstate,h/10);
			digitstate=DSTATE_2;
			break;
		
		case DSTATE_2:
			numOut(digitstate,h%10);
			digitstate=DSTATE_3;
			break;
			
		case DSTATE_3:
			PORTD&=~(1<<PD6);
			numOut(digitstate,m/10);
			digitstate=DSTATE_4;
			break;
			
		case DSTATE_4:
			numOut(digitstate,m%10);
			digitstate=DSTATE_OFF;
			break;
		case DSTATE_OFF:
			OCR0A=LED_OFF_TICKS;
			blank();
			digitstate=DSTATE_1;
			break;
	}
		
		
}